@@ -24,6 +24,9 @@
 
 #include <glib.h>
 
+gboolean
+_pango_Is_Emoji_Base_Character (gunichar ch);
+
 gboolean
 _pango_Is_Emoji_Extended_Pictographic (gunichar ch);
 
@@ -37,18 +40,17 @@ struct _PangoEmojiIter
   const gchar *end;
   gboolean is_emoji;
 
-  const gchar *token_start;
-  const gchar *token_end;
-
-  const unsigned char *types;
+  unsigned char *types;
+  unsigned char types_[64];
   unsigned int n_chars;
   unsigned int cursor;
 };
 
 PangoEmojiIter *
 _pango_emoji_iter_init (PangoEmojiIter *iter,
-			const char     *text,
-			int             length);
+                        const char     *text,
+                        int             length,
+                        unsigned int    n_chars);
 
 gboolean
 _pango_emoji_iter_next (PangoEmojiIter *iter);
