@@ -25,8 +25,8 @@
 #include <pango/pangofc-fontmap.h>
 #include <pango/pangofc-decoder.h>
 #include <pango/pangofc-font-private.h>
-#include <pango/pango-fontmap-private.h>
-#include <pango/pango-fontset-private.h>
+#include <pango/pango-fontmap.h>
+#include <pango/pango-fontset.h>
 
 #include <fontconfig/fontconfig.h>
 
@@ -191,6 +191,9 @@ struct _PangoFcFontMapClass
   void (*_pango_reserved4) (void);
 };
 
+PangoFontDescription *font_description_from_pattern (FcPattern *pattern,
+                                                     gboolean   include_size,
+                                                     gboolean   shallow);
 
 G_END_DECLS
 
