@@ -3,13 +3,46 @@
 
 import os
 import re
-import toml
+
+toml_module = None
+try:
+    import tomllib as toml_lib
+    toml_module = 'tomlib'
+except ImportError:
+    try:
+        import tomli as toml_lib
+        toml_module = 'tomli'
+    except ImportError:
+        import toml as toml_lib
+        toml_module = 'toml'
 
 from urllib.parse import urljoin
+from packaging import version as packaging_version
 
 from . import core, log, utils
 
 
+class TomlConfig:
+    """Wrapper class for TOML loading"""
+
+    @staticmethod
+    def load(toml):
+        log.debug(f"Using TOML module: {toml_module}")
+        if toml_module is None:
+            log.error("No toml module found")
+        elif toml_module in ['tomlib', 'tomli']:
+            try:
+                with open(toml, "rb") as f:
+                    return toml_lib.load(f)
+            except toml_lib.TOMLDecodeError as err:
+                log.error(f"Invalid configuration file: {toml}: {err}")
+        elif toml_module in ['toml']:
+            try:
+                return toml_lib.load(toml)
+            except toml_lib.TomlDecodeError as err:
+                log.error(f"Invalid configuration file: {toml}: {err}")
+
+
 class GIDocConfig:
     """Load and represent the configuration for gidocgen"""
     def __init__(self, config_file=None):
@@ -17,11 +50,8 @@ class GIDocConfig:
 
         self._config = {}
         if self._config_file is not None:
-            try:
-                log.debug(f"Reading configuration file: {self._config_file}")
-                self._config = toml.load(self._config_file)
-            except toml.TomlDecodeError as err:
-                log.error(f"Invalid configuration file: {self._config_file}: {err}")
+            log.debug(f"Reading configuration file: {self._config_file}")
+            self._config = TomlConfig.load(self._config_file)
 
     @property
     def library(self):
@@ -214,19 +244,8 @@ class GIDocConfig:
         cur_version = self.library.get('version')
         if cur_version is None:
             return False
-        library_version = cur_version.split('.')
-        symbol_version = version.split('.')
-        if len(library_version) < 2 or len(symbol_version) < 2:
-            return False
-        if int(symbol_version[0]) > int(library_version[0]):
-            return True
-        elif int(symbol_version[0]) == int(library_version[0]):
-            if int(symbol_version[1]) > int(library_version[1]):
-                return True
-            else:
-                return False
-        else:
-            return False
+
+        return packaging_version.parse(version) > packaging_version.parse(cur_version)
 
     @property
     def generator(self):
@@ -241,11 +260,8 @@ class GITemplateConfig:
         self._config_file = os.path.join(templates_dir, template_name, f"{template_name}.toml")
 
         self._config = {}
-        try:
-            log.debug(f"Reading template configuration file: {self._config_file}")
-            self._config = toml.load(self._config_file)
-        except toml.TomlDecodeError as err:
-            log.error(f"Invalid template configuration file: {self._config_file}: {err}")
+        log.debug(f"Reading template configuration file: {self._config_file}")
+        self._config = TomlConfig.load(self._config_file)
 
     @property
     def name(self):
