@@ -85,7 +85,7 @@ class GirParser:
         """Prepend a path to the list of search paths"""
         self._search_paths = [path] + self._search_paths
 
-    def parse(self, girfile: T.TextIO) -> None:
+    def parse(self, girfile: T.Union[T.TextIO, str]) -> None:
         """Parse @girfile"""
         log.debug(f"Loading GIR for {girfile}")
         tree = ET.parse(girfile)
@@ -333,8 +333,7 @@ class GirParser:
         if deprecated is not None:
             deprecated_since = node.attrib.get('deprecated-version')
             deprecated_doc = self._maybe_parse_deprecated_doc(node)
-            if deprecated_doc is not None:
-                element.set_deprecated(deprecated_doc, deprecated_since)
+            element.set_deprecated(deprecated_doc, deprecated_since)
 
     def _parse_array(self, node: ET.Element) -> ast.Type:
         child = node.find('core:array', GI_NAMESPACES)
